/*=============================================================================
 * parser for CSP instances represented in XML format
 * 
 * Copyright (c) 2006 Olivier ROUSSEL (olivier.roussel <at> cril.univ-artois.fr)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *=============================================================================
 */
#include <stdio.h>

#include "C_CSPParserCallback.h"

void beginInstance(void *solverData, const char *name)
{
  printf("beginInstance(name=\"%s\")\n",name);
}

void beginDomainsSection(void *solverData, int nbDomains) 
{  
  printf("beginDomainsSection(nbDomains=%d)\n",nbDomains);
}

void beginDomain(void *solverData, const char *name, int idDomain, int nbValue) 
{
  printf("  beginDomain(name=\"%s\",idDomain=%d,nbValue=%d)\n",name,idDomain,nbValue);
}

void addDomainValue(void *solverData, int v) 
{
  printf("    addDomainValue(v=%d)\n",v);
}

void addDomainValueInterval(void *solverData, int first,int last) 
{
  printf("    addDomainValueInterval(first=%d,last=%d)\n",first,last);
}

void endDomain(void *solverData) 
{
  printf("  endDomain()\n");
}

void endDomainsSection(void *solverData) 
{
  printf("endDomainsSection()\n");
}


void beginVariablesSection(void *solverData, int nbVariables) 
{
  printf("beginVariablesSection(nbVariables=%d)\n",nbVariables);
}

void addVariable(void *solverData, const char *name, int idVar,
		 const char *domain, int idDomain) 
{
  printf("  addVariable(name=\"%s\",idVar=%d,domain=\"%s\",idDomain=%d)\n",
	 name,idVar,domain,idDomain);
}

void endVariablesSection(void *solverData) 
{
  printf("endVariablesSection()\n");
}


void beginRelationsSection(void *solverData, int nbRelations) 
{
  printf("beginRelationsSection(nbRelations=%d)\n",nbRelations);
}


void beginRelation(void *solverData, const char *name, int idRel,
		   int arity, int nbTuples, int isSupport) 
{
  printf("  beginRelation(name=\"%s\",idRel=%d,arity=%d,nbTuples=%d,isSupport=%d)\n",
	 name,idRel,arity,nbTuples,isSupport);
}

void addRelationTuple(void *solverData, int arity, int tuple[]) 
{
  int i;
  printf("    addRelationTuple(arity=%d,tuple=[%d",arity,tuple[0]);
  for(i=1;i<arity;++i)
    printf(",%d",tuple[i]);
  printf("])\n");
}

void endRelation(void *solverData) 
{
  printf("  endRelation()\n");
}

void endRelationsSection(void *solverData) 
{
  printf("endRelationsSection()\n");
}

void beginPredicatesSection(void *solverData, int nbPredicates) 
{
  printf("beginPredicatesSection(nbPredicates=%d)\n",nbPredicates);
}

void beginPredicate(void *solverData, const char *name, int idPred) 
{
  printf("  beginPredicate(name=\"%s\",idPred=%d)\n",name,idPred);
}

void addFormalParameter(void *solverData, int pos, const char *name, const char *type) 
{
  printf("    addFormalParameter(pos=%d,name=\"%s\",type=\"%s\")\n",
	 pos,name,type);
}

void predicateExpressionAST(void *solverData, C_AST *tree) 
{
  printf("    predicateExpressionAST(C_AST *tree)\n");
}

void predicateExpression(void *solverData, const char *expr) 
{
  printf("    predicateExpression(expr=\"%s\")\n",expr);
}

void endPredicate(void *solverData) 
{
  printf("  endPredicate()\n");
}

void endPredicatesSection(void *solverData) 
{
  printf("endPredicatesSection()\n");
}

void beginConstraintsSection(void *solverData, int nbConstraints) 
{
  printf("beginConstraintsSection(nbConstraints=%d)\n",nbConstraints);
}

void beginConstraint(void *solverData, const char *name, int idConstr,
		     int arity, 
		     const char *reference, 
		     int type, int id) 
{
  printf("  beginConstraint(name=\"%s\",idConstr=%d,arity=%d,reference=\"%s\",type=%d,id=%d)\n",
	 name,idConstr,arity,reference,type,id);
}

void addVariableToConstraint(void *solverData, const char *name, int idVar) 
{
  printf("    addVariableToConstraint(name=\"%s\",idVar=%d)\n",name,idVar);
}

void addEffectiveParameter(void *solverData, int pos, const char *expr) 
{
  printf("    addEffectiveParameter(pos=%d,expr=\"%s\")\n",pos,expr);
}

void addEffectiveParameterAST(void *solverData, int pos, C_AST *tree) 
{
  printf("    addEffectiveParameterAST(pos=%d, C_AST *tree)\n",pos);
}

void addEffectiveParameterVariable(void *solverData, int pos, const char *name, int idVar) 
{
  printf("    addEffectiveParameterVariable(pos=%d,name=\"%s\",idVar=%d)\n",
	 pos,name,idVar);
}

void addEffectiveParameterInteger(void *solverData, int pos, int value)
{
  printf("    addEffectiveParameterInteger(pos=%d,value=%d)\n",pos,value);
}

void addEffectiveParameterList(void *solverData, int pos) 
{
  printf("    addEffectiveParameterList(pos=%d)\n",pos);
}

void beginList(void *solverData) 
{
  printf("      beginList()\n");
}

void addListVariable(void *solverData, int pos, const char *name, int idVar) 
{
  printf("        addListVariable(pos=%d,name=\"%s\",idVar=%d)\n",
	 pos,name,idVar);
}

void addListInteger(void *solverData, int pos, int value) 
{
  printf("        addListInteger(pos=%d,value=%d)\n",pos,value);
}

void endList(void *solverData) 
{
  printf("      endList()\n");
}

void constraintExpressionAST(void *solverData, C_AST *tree) 
{
  printf("    constraintExpressionAST(C_AST *tree)\n");
}

void constraintExpression(void *solverData, const char *expr) 
{
  printf("    constraintExpression(\"expr=%s\")\n",expr);
}

void endConstraint(void *solverData) 
{
  printf("  endConstraint()\n");
}

void endConstraintsSection(void *solverData) 
{
  printf("endConstraintsSection()\n");
}

void endInstance(void *solverData) 
{
  printf("endInstance()\n");
}

